// -*- coding: iso-8859-1-unix -*-

#include <check.hpp>
#include <configuration.h>
#include <openbus.hpp>

int main(int argc, char** argv)
{
  openbus::log().set_level(openbus::debug_level);
  openbus::configuration cfg(argc, argv);
  std::auto_ptr<openbus::orb_ctx>
    orb_ctx(openbus::ORBInitializer(argc, argv));
  CORBA::Object_var
    obj(orb_ctx->orb()->resolve_initial_references("OpenBusContext"));
  openbus::OpenBusContext
    *bus_ctx(dynamic_cast<openbus::OpenBusContext *>(obj.in()));

  boost::shared_ptr<openbus::Connection> conn_A(
    bus_ctx->connectByAddress(cfg.host(), cfg.port()));
  conn_A->loginByPassword("A", "A", cfg.domain());

  boost::shared_ptr<openbus::Connection> conn_B(
    bus_ctx->connectByAddress(cfg.host(), cfg.port()));
  conn_B->loginByPassword("B", "B", cfg.domain());

  boost::shared_ptr<openbus::Connection> conn_C(
    bus_ctx->connectByAddress(cfg.host(), cfg.port()));
  conn_C->loginByPassword("C", "C", cfg.domain());

  bus_ctx->setDefaultConnection(conn_A);

  openbus::CallerChain chain_A_B(
    bus_ctx->makeChainFor(conn_B->login()->entity.in()));
  bus_ctx->setCurrentConnection(conn_B);
  bus_ctx->joinChain(chain_A_B);

  openbus::CallerChain chain_A_B_C(
    bus_ctx->makeChainFor(conn_C->login()->entity.in()));

  if ("C" != chain_A_B_C.target())
  {
    std::cerr << "'C' != chain_A_B_C.target()" << std::endl;
    std::abort();
  }

  if ("B" != std::string(chain_A_B_C.caller().entity.in()))
  {
    std::cerr << "'B' != std::string(chain_A_B_C.caller().entity.in())"
              << std::endl;
    std::abort();
  }

  if (conn_B->login()->id.in() != std::string(chain_A_B_C.caller().id.in()))
  {
    std::cerr << "conn_B->login()->id.in() \
                  != std::string(chain_A_B_C.caller().id.in())"
              << std::endl;
    std::abort();
  }

  if (chain_A_B_C.originators().length() != 1)
  {
    std::cerr << "chain_A_B_C.originators().length() != 1" << std::endl;
    std::abort();
  }

  if ("A" != std::string(chain_A_B_C.originators()[0u].entity.in()))
  {
    std::cerr << "'A' != std::string( \
                  chain_A_B_C.originators()[0u].entity.in())"
              << std::endl;
    std::abort();
  }

  if (conn_A->login()->id.in()
      != std::string(chain_A_B_C.originators()[0u].id.in()))
  {
    std::cerr << "conn_B->login()->id.in() \
                  != std::string(chain_A_B_C.originators()[0u].id.in())"
              << std::endl;
    std::abort();
  }
  return 0; //MSVC
}
